#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Processor::Scheduler::Default module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 18;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Processor::Scheduler::Default';

        use_ok('Tester');
        use_ok($TESTED_CLASS);

        $ENV{PATH} = '/bin:/usr/bin';
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
### $Mentat::Processor::Scheduler::Default::VERSION

### Attempting to create Mentat::Processor::Scheduler::Default instance...
my $instance = $TESTED_CLASS->new();
isa_ok($instance, $TESTED_CLASS);

### Viewing the created instance: $instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

my @RESULT = ();
my @PLAN = (
        ['schedule', 'event-a01'],
        ['schedule', 'event-a02'],
        ['schedule', 'event-a03'],

        ['schedule_first', 'event-b01'],
        ['schedule_first', 'event-b02'],
        ['schedule_first', 'event-b03'],

        ['schedule_at', 'event-c01', time+6],
        ['schedule_at', 'event-c02', time+4],
        ['schedule_at', 'event-c03', time],

        ['dump', $instance],
        ['count'],
        ['count',1],

        ['cancel_event', 1000],
        ['cancel_event', 7],
        ['cancel_event', 2],

        ['dump', $instance],
        ['count'],
        ['count',1],

        ['next_event'],
        ['sleep',1],
        ['next_event'],
        ['sleep',1],
        ['next_event'],
        ['sleep',1],
        ['next_event'],
        ['sleep',1],
        ['next_event'],
        ['sleep',1],
        ['next_event'],
        ['sleep',1],
        ['next_event'],
        ['sleep',1],
        ['next_event'],
        ['sleep',1],
        ['next_event'],
        ['sleep',1],
        ['next_event'],

        ['count'],
    );

my ($action, @args);
foreach my $action_desc (@PLAN) {
    ($action, @args) = @$action_desc;
    if ($action eq 'sleep') {
        sleep(@args);
        push(@RESULT, "Sleep: @args");
    }
    elsif ($action eq 'dump') {
        push(@RESULT, "Dump: " . Dumper(@args));
    }
    else {
        push(@RESULT, [$instance->$action(@args)]);
    }
}
my $STATISTICS = $instance->statistics();

diag("Result dump: " . Dumper(\@RESULT)) if Tester::D();
diag("Statistics dump: " . Dumper($STATISTICS)) if Tester::D();

is($RESULT[10]->[0],      9,           'Plan check test 01');
is($RESULT[11]->[0],      9,           'Plan check test 02');
is($RESULT[16]->[0],      9,           'Plan check test 03');
is($RESULT[17]->[0],      7,           'Plan check test 04');
is($RESULT[18]->[0]->[1], 'event-c03', 'Plan check test 05');
is($RESULT[20]->[0]->[1], 'event-b03', 'Plan check test 06');
is($RESULT[22]->[0]->[1], 'event-b02', 'Plan check test 07');
is($RESULT[24]->[0]->[1], 'event-b01', 'Plan check test 08');
is($RESULT[26]->[0]->[1], 'event-c02', 'Plan check test 09');
is($RESULT[28]->[0]->[1], 'event-a01', 'Plan check test 10');
is($RESULT[30]->[0]->[1], 'event-a03', 'Plan check test 11');
is($RESULT[32]->[0]->[1], undef,       'Plan check test 12');
is($RESULT[34]->[0]->[1], undef,       'Plan check test 13');
is($RESULT[36]->[0]->[1], undef,       'Plan check test 14');
is($RESULT[37]->[0],      0,           'Plan check test 15');
